#ifndef NODEFASTBIT_H
#define NOEDFASTBIT_H

#include <string>
#include <map>
#include <utility>
#include <node.h>
#include <node_object_wrap.h>
#include "import/fastbit-2.0.3/src/capi.h"

using namespace v8;

class NodeFastbit : public node::ObjectWrap{
public:
    static void Init( Handle<Object> exports );

private:
    explicit NodeFastbit();
    ~NodeFastbit();

    static void New( const FunctionCallbackInfo<Value>& args );
    static Persistent<Function> constructor;

    // api methods
    static void build_indexes( const FunctionCallbackInfo<Value>& args );
    static void purge_indexes( const FunctionCallbackInfo<Value>& args );
    static void build_index( const FunctionCallbackInfo<Value>& args );
    static void purge_index( const FunctionCallbackInfo<Value>& args );
    static void reorder_partition( const FunctionCallbackInfo<Value>& args );

    static void build_query( const FunctionCallbackInfo<Value>& args );
    static void destroy_query( const FunctionCallbackInfo<Value>& args );
    static void get_result_columns( const FunctionCallbackInfo<Value>& args );
    static void get_result_rows( const FunctionCallbackInfo<Value>& args );
    static void get_result_row_ids( const FunctionCallbackInfo<Value>& args );
    static void get_select_clause( const FunctionCallbackInfo<Value>& args );
    static void get_from_clause( const FunctionCallbackInfo<Value>& args );
    static void get_where_clause( const FunctionCallbackInfo<Value>& args );

    static void get_qualified_floats( const FunctionCallbackInfo<Value>& args );
    static void get_qualified_doubles( const FunctionCallbackInfo<Value>& args );
    static void get_qualified_bytes( const FunctionCallbackInfo<Value>& args );
    static void get_qualified_shorts( const FunctionCallbackInfo<Value>& args );
    static void get_qualified_ints( const FunctionCallbackInfo<Value>& args );
    static void get_qualified_longs( const FunctionCallbackInfo<Value>& args );
    static void get_qualified_ubytes( const FunctionCallbackInfo<Value>& args );
    static void get_qualified_ushorts( const FunctionCallbackInfo<Value>& args );
    static void get_qualified_uints( const FunctionCallbackInfo<Value>& args );
    static void get_qualified_ulongs( const FunctionCallbackInfo<Value>& args );
    static void get_qualified_strings( const FunctionCallbackInfo<Value>& args );

    static void build_result_set( const FunctionCallbackInfo<Value>& args );
    static void destroy_result_set( const FunctionCallbackInfo<Value>& args );
    static void result_set_next( const FunctionCallbackInfo<Value>& args );
    static void result_set_next_bundle( const FunctionCallbackInfo<Value>& args );
    static void result_set_get_int( const FunctionCallbackInfo<Value>& args );
    static void result_set_get_unsigned( const FunctionCallbackInfo<Value>& args );
    static void result_set_get_long( const FunctionCallbackInfo<Value>& args );
    static void result_set_get_float( const FunctionCallbackInfo<Value>& args );
    static void result_set_get_double( const FunctionCallbackInfo<Value>& args );
    static void result_set_get_string( const FunctionCallbackInfo<Value>& args );
    static void result_set_getInt( const FunctionCallbackInfo<Value>& args );
    static void result_set_getUnsigned( const FunctionCallbackInfo<Value>& args );
    static void result_set_getLong( const FunctionCallbackInfo<Value>& args );
    static void result_set_getFloat( const FunctionCallbackInfo<Value>& args );
    static void result_set_getDouble( const FunctionCallbackInfo<Value>& args );
    static void result_set_getString( const FunctionCallbackInfo<Value>& args );
    static void flush_buffer( const FunctionCallbackInfo<Value>& args );
    static void add_values( const FunctionCallbackInfo<Value>& args );
    static void rows_in_partition( const FunctionCallbackInfo<Value>& args );
    static void columns_in_partition( const FunctionCallbackInfo<Value>& args );

    static void init( const FunctionCallbackInfo<Value>& args );
    static void cleanup( const FunctionCallbackInfo<Value>& args );
    static void set_verbose_level( const FunctionCallbackInfo<Value>& args );
    static void get_verbose_level( const FunctionCallbackInfo<Value>& args );
    static void set_logfile( const FunctionCallbackInfo<Value>& args );
    static void get_logfile( const FunctionCallbackInfo<Value>& args );
    static void get_logfilepointer( const FunctionCallbackInfo<Value>& args );
    static void read_clock( const FunctionCallbackInfo<Value>& args );
    static void get_version_string( const FunctionCallbackInfo<Value>& args );
    static void get_version_number( const FunctionCallbackInfo<Value>& args );

private:
    static char *getChar( Local<Value> value, const char *fallback = "" );
    static bool isWrongArgNumber( const FunctionCallbackInfo<Value>& args,
                                    int argNumExpected );
    static bool isWrongType( bool predicateRes, int argId );


private:
// member vars TODO better handle handling :D
  FILE* m_logFileHandle;

  std::map< std::string, FastBitQuery* > m_queryHandles;
  std::map< std::string, FastBitResultSet* > m_resultSetHandles;
};

#endif
